//
// Time Zone
//

(function(jef, i18n, string) {

	if ( jef.isDef(i18n.TimeZone) )
		return;

	/**
	 * TimeZone class implemented a time zone resolution and name information
	 * source for client applications. The time zone object is initiated from a
	 * time zone information object. Application can initiate a time zone
	 * statically, or it may choose to initiate from a data obtained from
	 * server. Each time zone information array is small, but the whole set of
	 * data is too much for client application to download. If end user is
	 * allowed to change time zone setting, dynamic retrieval should be the
	 * method to use. In case only time zone offset is known, there is a decent
	 * fallback that only use the time zone offset to create a TimeZone object.
	 * A whole set of time zone information array was available under
	 * http://go/js_locale_data. It is generated based on CLDR and Olson time
	 * zone data base (through pytz), and will be updated timely.
	 * 
	 * @constructor
	 */
	i18n.TimeZone = function() {
		/**
		 * The standard time zone id.
		 * 
		 * @type {string}
		 * @private
		 */
		this.timeZoneId_;

		/**
		 * The standard, non-daylight time zone offset.
		 * 
		 * @type {number}
		 * @private
		 */
		this.standardOffset_;

		/**
		 * An array of string that can have 2 or 4 elements, long and short
		 * names for standard time zone, and long and short names for daylight
		 * time zone if it has daylight time transitions.
		 * 
		 * @type {Array.<string>}
		 * @private
		 */
		this.tzNames_;

		/**
		 * Daylight/standard time transition array. It lists transition points
		 * since 1970 until some year in future. It always in pair of
		 * (transition point) + (time zone offset adjustment)
		 * 
		 * @type {Array.<number>}
		 * @private
		 */
		this.transitions_;
	};

	/**
	 * Milliseconds per hour constant.
	 * 
	 * @type {number}
	 * @private
	 */
	i18n.TimeZone.MILLISECONDS_PER_HOUR_ = 3600 * 1000;

	/**
	 * Enum of time zone names. The value will be used as index of in time zone
	 * name array.
	 * 
	 * @enum {number}
	 */
	i18n.TimeZone.NameType = {
		STD_SHORT_NAME : 0,
		STD_LONG_NAME : 1,
		DLT_SHORT_NAME : 2,
		DLT_LONG_NAME : 3
	};

	/**
	 * This factory method creates a time zone instance. It takes either a time
	 * zone information array or a simple timezone offset. The latter form does
	 * not offer the same set of functionalities as first form.
	 * 
	 * @param {number|Object}
	 *            timeZoneData this parameter could take 2 types, if it is a
	 *            number, a simple TimeZone object will be created. Otherwise,
	 *            it should be an Object that holds all time zone related
	 *            information.
	 * @return {jef.i18n.TimeZone} A jef.i18n.TimeZone object for the given time
	 *         zone data.
	 */
	i18n.TimeZone.createTimeZone = function(timeZoneData) {
		if (typeof timeZoneData == 'number') {
			return i18n.TimeZone.createSimpleTimeZone_(timeZoneData);
		}
		var tz = new i18n.TimeZone();
		tz.timeZoneId_ = timeZoneData['id'];
		tz.standardOffset_ = -timeZoneData['std_offset'];
		tz.tzNames_ = timeZoneData['names'];
		tz.transitions_ = timeZoneData['transitions'];
		return tz;
	};

	/**
	 * This factory method provides a decent fallback to create a time zone
	 * object just based on a given time zone offset.
	 * 
	 * @param {number}
	 *            timeZoneOffsetInMinutes time zone offset in minutes.
	 * @return {jef.i18n.TimeZone} A jef.i18n.TimeZone object generated by just
	 *         using the time zone offset information.
	 * @private
	 */
	i18n.TimeZone.createSimpleTimeZone_ = function(timeZoneOffsetInMinutes) {
		var tz = new i18n.TimeZone();
		tz.standardOffset_ = timeZoneOffsetInMinutes;
		tz.timeZoneId_ = i18n.TimeZone
				.composePosixTimeZoneID_(timeZoneOffsetInMinutes);
		var str = i18n.TimeZone.composeUTCString_(timeZoneOffsetInMinutes);
		tz.tzNames_ = [ str, str ];
		tz.transitions_ = [];
		return tz;
	};

	/**
	 * Generate GMT string given a time zone offset.
	 * 
	 * @param {number}
	 *            offset time zone offset in minutes.
	 * @return {string} GMT string for this offset.
	 * @private
	 */
	i18n.TimeZone.composeGMTString_ = function(offset) {
		var parts = [ 'GMT' ];
		parts.push(offset <= 0 ? '+' : '-');
		offset = Math.abs(offset);
		parts.push(string.padNumber(Math.floor(offset / 60) % 100, 2), ':',
				string.padNumber(offset % 60, 2));
		return parts.join('');
	};

	/**
	 * POSIX time zone ID as fallback.
	 * 
	 * @param {number}
	 *            offset time zone offset in minutes.
	 * @return {string} posix time zone id for given offset.
	 * @private
	 */
	i18n.TimeZone.composePosixTimeZoneID_ = function(offset) {
		if (offset == 0) {
			return 'Etc/GMT';
		}
		var parts = [ 'Etc/GMT', offset < 0 ? '-' : '+' ];
		offset = Math.abs(offset);
		parts.push(Math.floor(offset / 60) % 100);
		offset = offset % 60;
		if (offset != 0) {
			parts.push(':', string.padNumber(offset, 2));
		}
		return parts.join('');
	};

	/**
	 * Generate UTC string.
	 * 
	 * @param {number}
	 *            offset time zone offset in minutes.
	 * @return {string} UTC string for given offset.
	 * @private
	 */
	i18n.TimeZone.composeUTCString_ = function(offset) {
		if (offset == 0) {
			return 'UTC';
		}
		var parts = [ 'UTC', offset < 0 ? '+' : '-' ];
		offset = Math.abs(offset);
		parts.push(Math.floor(offset / 60) % 100);
		offset = offset % 60;
		if (offset != 0) {
			parts.push(':', offset);
		}
		return parts.join('');
	};

	/**
	 * Return the adjustment amount of time zone offset. When daylight time is
	 * in effect, this number will be positive. Otherwise, it is zero.
	 * 
	 * @param {Date}
	 *            date the time to check.
	 * @return {number} offset amount.
	 */
	i18n.TimeZone.prototype.getDaylightAdjustment = function(date) {
		var timeInMs = Date.UTC(date.getUTCFullYear(), date.getUTCMonth(), date
				.getUTCDate(), date.getUTCHours(), date.getUTCMinutes());
		var timeInHours = timeInMs / i18n.TimeZone.MILLISECONDS_PER_HOUR_;
		var index = 0;
		while (index < this.transitions_.length
				&& timeInHours >= this.transitions_[index]) {
			index += 2;
		}
		return (index == 0) ? 0 : this.transitions_[index - 1];
	};

	/**
	 * Return the GMT representation of this time zone object.
	 * 
	 * @param {Date}
	 *            date The date for which time to retrieve GMT string.
	 * @return {string} GMT representation string.
	 */
	i18n.TimeZone.prototype.getGMTString = function(date) {
		return i18n.TimeZone.composeGMTString_(this.getOffset(date));
	};

	/**
	 * To get long time zone name for given date.
	 * 
	 * @param {Date}
	 *            date The Date object for which time to retrieve long time zone
	 *            name.
	 * @return {string} long time zone name.
	 */
	i18n.TimeZone.prototype.getLongName = function(date) {
		return this.tzNames_[this.isDaylightTime(date) ? i18n.TimeZone.NameType.DLT_LONG_NAME
				: i18n.TimeZone.NameType.STD_LONG_NAME];
	};

	/**
	 * To get time zone offset (in minutes) relative to UTC for given date. To
	 * be consistent with JDK/Javascript API, west of Greenwich will be
	 * positive.
	 * 
	 * @param {Date}
	 *            date The date for which time to retrieve time zone offset.
	 * @return {number} time zone offset in minutes.
	 */
	i18n.TimeZone.prototype.getOffset = function(date) {
		return this.standardOffset_ - this.getDaylightAdjustment(date);
	};

	/**
	 * To get RFC representation of certain time zone name for given date.
	 * 
	 * @param {Date}
	 *            date The Date object for which time to retrieve RFC time zone
	 *            string.
	 * @return {string} RFC time zone string.
	 */
	i18n.TimeZone.prototype.getRFCTimeZoneString = function(date) {
		var offset = -this.getOffset(date);
		var parts = [ offset < 0 ? '-' : '+' ];
		offset = Math.abs(offset);
		parts.push(string.padNumber(Math.floor(offset / 60) % 100, 2),
				string.padNumber(offset % 60, 2));
		return parts.join('');
	};

	/**
	 * To get short time zone name for given date.
	 * 
	 * @param {Date}
	 *            date The date for which time to retrieve short time zone.
	 * @return {string} short time zone name.
	 */
	i18n.TimeZone.prototype.getShortName = function(date) {
		return this.tzNames_[this.isDaylightTime(date) ? i18n.TimeZone.NameType.DLT_SHORT_NAME
				: i18n.TimeZone.NameType.STD_SHORT_NAME];
	};

	/**
	 * Return time zone id for this time zone.
	 * 
	 * @return {string} time zone id.
	 */
	i18n.TimeZone.prototype.getTimeZoneId = function() {
		return this.timeZoneId_;
	};

	/**
	 * Check if the given time fall within daylight saving period.
	 * 
	 * @param {Date}
	 *            date time for which to check.
	 * @return {boolean} true if daylight time in effect.
	 */
	i18n.TimeZone.prototype.isDaylightTime = function(date) {
		return this.getDaylightAdjustment(date) > 0;
	};

})(jef, jef.i18n, jef.string);

//
// Date Time Symbols
//

(function(jef, i18n) {

	if ( i18n.getLocaleUpdate('java-datetime-symbols') )
		return;

	i18n.DateTimeSymbols_es_CL = {
		ERAS : [ 'a.C.', 'd.C.' ],
		ERANAMES : [ 'antes de Cristo', 'anno Dómini' ],
		NARROWMONTHS : [ 'E', 'F', 'M', 'A', 'M', 'J', 'J', 'A', 'S', 'O', 'N',
				'D' ],
		STANDALONENARROWMONTHS : [ 'E', 'F', 'M', 'A', 'M', 'J', 'J', 'A', 'S',
				'O', 'N', 'D' ],
		MONTHS : [ 'enero', 'febrero', 'marzo', 'abril', 'mayo', 'junio',
				'julio', 'agosto', 'septiembre', 'octubre', 'noviembre',
				'diciembre' ],
		STANDALONEMONTHS : [ 'enero', 'febrero', 'marzo', 'abril', 'mayo',
				'junio', 'julio', 'agosto', 'septiembre', 'octubre',
				'noviembre', 'diciembre' ],
		SHORTMONTHS : [ 'ene', 'feb', 'mar', 'abr', 'may', 'jun', 'jul', 'ago',
				'sep', 'oct', 'nov', 'dic' ],
		STANDALONESHORTMONTHS : [ 'ene', 'feb', 'mar', 'abr', 'may', 'jun',
				'jul', 'ago', 'sep', 'oct', 'nov', 'dic' ],
		WEEKDAYS : [ 'domingo', 'lunes', 'martes', 'miércoles', 'jueves',
				'viernes', 'sábado' ],
		STANDALONEWEEKDAYS : [ 'domingo', 'lunes', 'martes', 'miércoles',
				'jueves', 'viernes', 'sábado' ],
		SHORTWEEKDAYS : [ 'dom', 'lun', 'mar', 'mié', 'jue', 'vie', 'sáb' ],
		STANDALONESHORTWEEKDAYS : [ 'dom', 'lun', 'mar', 'mié', 'jue', 'vie',
				'sáb' ],
		NARROWWEEKDAYS : [ 'D', 'L', 'M', 'M', 'J', 'V', 'S' ],
		STANDALONENARROWWEEKDAYS : [ 'D', 'L', 'M', 'M', 'J', 'V', 'S' ],
		SHORTQUARTERS : [ 'T1', 'T2', 'T3', 'T4' ],
		QUARTERS : [ '1er trimestre', '2º trimestre', '3er trimestre',
				'4º trimestre' ],
		AMPMS : [ 'a.m.', 'p.m.' ],
		DATEFORMATS : [ 'EEEE d \'de\' MMMM \'de\' y',
				'd \'de\' MMMM \'de\' y', 'dd-MM-yyyy', 'dd-MM-yy' ],
		TIMEFORMATS : [ 'HH:mm:ss zzzz', 'H:mm:ss z', 'H:mm:ss', 'H:mm' ],
		AVAILABLEFORMATS : {
			'Md' : 'd/M',
			'MMMMd' : 'd \'de\' MMMM',
			'MMMd' : 'd MMM'
		},
		FIRSTDAYOFWEEK : 0,
		WEEKENDRANGE : [ 5, 6 ],
		FIRSTWEEKCUTOFFDAY : 3
	};

	i18n.DateTimeSymbols_en = {
		ERAS : [ 'BC', 'AD' ],
		ERANAMES : [ 'Before Christ', 'Anno Domini' ],
		NARROWMONTHS : [ 'J', 'F', 'M', 'A', 'M', 'J', 'J', 'A', 'S', 'O', 'N',
				'D' ],
		STANDALONENARROWMONTHS : [ 'J', 'F', 'M', 'A', 'M', 'J', 'J', 'A', 'S',
				'O', 'N', 'D' ],
		MONTHS : [ 'January', 'February', 'March', 'April', 'May', 'June',
				'July', 'August', 'September', 'October', 'November',
				'December' ],
		STANDALONEMONTHS : [ 'January', 'February', 'March', 'April', 'May',
				'June', 'July', 'August', 'September', 'October', 'November',
				'December' ],
		SHORTMONTHS : [ 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug',
				'Sep', 'Oct', 'Nov', 'Dec' ],
		STANDALONESHORTMONTHS : [ 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
				'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec' ],
		WEEKDAYS : [ 'Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday',
				'Friday', 'Saturday' ],
		STANDALONEWEEKDAYS : [ 'Sunday', 'Monday', 'Tuesday', 'Wednesday',
				'Thursday', 'Friday', 'Saturday' ],
		SHORTWEEKDAYS : [ 'Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat' ],
		STANDALONESHORTWEEKDAYS : [ 'Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri',
				'Sat' ],
		NARROWWEEKDAYS : [ 'S', 'M', 'T', 'W', 'T', 'F', 'S' ],
		STANDALONENARROWWEEKDAYS : [ 'S', 'M', 'T', 'W', 'T', 'F', 'S' ],
		SHORTQUARTERS : [ 'Q1', 'Q2', 'Q3', 'Q4' ],
		QUARTERS : [ '1st quarter', '2nd quarter', '3rd quarter', '4th quarter' ],
		AMPMS : [ 'AM', 'PM' ],
		DATEFORMATS : [ 'EEEE, MMMM d, y', 'MMMM d, y', 'MMM d, y', 'M/d/yy' ],
		TIMEFORMATS : [ 'h:mm:ss a zzzz', 'h:mm:ss a z', 'h:mm:ss a', 'h:mm a' ],
		AVAILABLEFORMATS : {
			'Md' : 'M/d',
			'MMMMd' : 'MMMM d',
			'MMMd' : 'MMM d'
		},
		FIRSTDAYOFWEEK : 6,
		WEEKENDRANGE : [ 5, 6 ],
		FIRSTWEEKCUTOFFDAY : 2
	};

	// Define en_US for String format
	i18n.DateTimeSymbols_en_US = i18n.DateTimeSymbols_en;
	
	// Add locale update function and invoke it
	i18n.addLocaleUpdate('java-datetime-symbols', function(locale){	
	
		var symbols = i18n['DateTimeSymbols_' + locale];
		if(symbols){
			i18n.DateTimeSymbols = symbols;
			return;
		}
		symbols = locale.length > 2 && i18n['DateTimeSymbols_' + locale.substr(0,2)];
		if(symbols){
			i18n.DateTimeSymbols = symbols;
			return;
		}
		// Default to Chile
		i18n.DateTimeSymbols = i18n.DateTimeSymbols_es_CL;
		
	})(i18n.getLocale());

})(jef, jef.i18n);

//
// Date Time Format
//

(function(jef, i18n, string) {

	if(jef.isDef(jef.date.JavaDateTimeFormat))
		return;
	

	/**
	 * Datetime formatting functions following the pattern specification as
	 * defined in JDK, ICU and CLDR, with minor modification for typical usage
	 * in JS. Pattern specification: (Refer to JDK/ICU/CLDR)
	 * 
	 * <pre>
	 * Symbol Meaning Presentation        Example
	 * ------   -------                 ------------        -------
	 * G        era designator          (Text)              AD
	 * y#       year                    (Number)            1996
	 * Y*       year (week of year)     (Number)            1997
	 * u*       extended year           (Number)            4601
	 * M        month in year           (Text &amp; Number)     July &amp; 07
	 * d        day in month            (Number)            10
	 * h        hour in am/pm (1&tilde;12)    (Number)            12
	 * H        hour in day (0&tilde;23)      (Number)            0
	 * m        minute in hour          (Number)            30
	 * s        second in minute        (Number)            55
	 * S        fractional second       (Number)            978
	 * E        day of week             (Text)              Tuesday
	 * e*       day of week (local 1&tilde;7) (Number)            2
	 * D*       day in year             (Number)            189
	 * F*       day of week in month    (Number)            2 (2nd Wed in July)
	 * w*       week in year            (Number)            27
	 * W*       week in month           (Number)            2
	 * a        am/pm marker            (Text)              PM
	 * k        hour in day (1&tilde;24)      (Number)            24
	 * K        hour in am/pm (0&tilde;11)    (Number)            0
	 * z        time zone               (Text)              Pacific Standard Time
	 * Z        time zone (RFC 822)     (Number)            -0800
	 * v        time zone (generic)     (Text)              Pacific Time
	 * g*       Julian day              (Number)            2451334
	 * A*       milliseconds in day     (Number)            69540000
	 * '        escape for text         (Delimiter)         'Date='
	 * ''       single quote            (Literal)           'o''clock'
	 * 
	 * Item marked with '*' are not supported yet.
	 * Item marked with '#' works different than java
	 * 
	 * The count of pattern letters determine the format.
	 * (Text): 4 or more, use full form, &lt;4, use short or abbreviated form if it
	 * exists. (e.g., &quot;EEEE&quot; produces &quot;Monday&quot;, &quot;EEE&quot; produces &quot;Mon&quot;)
	 * 
	 * (Number): the minimum number of digits. Shorter numbers are zero-padded to
	 * this amount (e.g. if &quot;m&quot; produces &quot;6&quot;, &quot;mm&quot; produces &quot;06&quot;). Year is handled
	 * specially; that is, if the count of 'y' is 2, the Year will be truncated to
	 * 2 digits. (e.g., if &quot;yyyy&quot; produces &quot;1997&quot;, &quot;yy&quot; produces &quot;97&quot;.) Unlike other
	 * fields, fractional seconds are padded on the right with zero.
	 * 
	 * (Text &amp; Number): 3 or over, use text, otherwise use number. (e.g., &quot;M&quot;
	 * produces &quot;1&quot;, &quot;MM&quot; produces &quot;01&quot;, &quot;MMM&quot; produces &quot;Jan&quot;, and &quot;MMMM&quot; produces
	 * &quot;January&quot;.)
	 * 
	 * Any characters in the pattern that are not in the ranges of ['a'..'z'] and
	 * ['A'..'Z'] will be treated as quoted text. For instance, characters like ':',
	 * '.', ' ', '#' and '@' will appear in the resulting time text even they are
	 * not embraced within single quotes.
	 * </pre>
	 * 
	 */

	/**
	 * Construct a DateTimeFormat object based on current locale.
	 * 
	 * @constructor
	 * @param {string|number} pattern The full define pattern or constant pattern number
	 * @param {object} options Object containing options {
	 * 		timeZone : {jef.i18n.TimeZone} The default time zone for the instance,
	 * 		locale : {string} locale key, ex: 'en', 'fr_FR', ...  (override default locale)            
	 * @see DateTimeFormat.Format The constants pattern number
	 */
	var DateTimeFormat = function(pattern, options){
		options = options || {};
		jef.assert(jef.isDef(pattern), 'Pattern must be defined');
		this.patternParts_ = [];
		this.dateTimeSymbols = i18n.DateTimeSymbols;
		if (jef.isDefAndNotNull(options.locale)) {
			var symbolsName = 'DateTimeSymbols_' + options.locale;
			var symbols = i18n[symbolsName];
			jef.assert(jef.isDef(symbols),
					"Date time symbols aren't defined for locale : "
							+ options.locale);
			this.dateTimeSymbols = symbols;
		}
		this.default_timeZone = options.timeZone;
		if (typeof pattern == 'number') {
			this.applyStandardPattern_(pattern);
		} else {
			this.applyPattern_(pattern);
		}
	};

	/**
	 * Enum to identify predefined Date/Time format pattern.
	 * 
	 * @enum {number}
	 */
	DateTimeFormat.Format = {
		FULL_DATE : 0,
		LONG_DATE : 1,
		MEDIUM_DATE : 2,
		SHORT_DATE : 3,
		FULL_TIME : 4,
		LONG_TIME : 5,
		MEDIUM_TIME : 6,
		SHORT_TIME : 7,
		FULL_DATETIME : 8,
		LONG_DATETIME : 9,
		MEDIUM_DATETIME : 10,
		SHORT_DATETIME : 11
	};

	/**
	 * regular expression pattern for parsing pattern string
	 * 
	 * @type {Array.<RegExp>}
	 * @private
	 */
	DateTimeFormat.TOKENS_ = [
	// quote string
			/^\'(?:[^\']|\'\')*\'/,
			// pattern chars
			/^(?:G+|y+|M+|k+|S+|E+|a+|h+|K+|H+|c+|L+|Q+|d+|m+|s+|v+|z+|Z+)/,
			// and all the other chars
			/^[^\'GyMkSEahKHcLQdmsvzZ]+/ // and all the other chars
	];

	/**
	 * These are token types, corresponding to above token definitions.
	 * 
	 * @enum {number}
	 * @private
	 */
	DateTimeFormat.PartTypes_ = {
		QUOTED_STRING : 0,
		FIELD : 1,
		LITERAL : 2
	};

	/**
	 * Apply specified pattern to this formatter object.
	 * 
	 * @param {string}
	 *            pattern String specifying how the date should be formatted.
	 * @private
	 */
	DateTimeFormat.prototype.applyPattern_ = function(pattern) {
		// lex the pattern, once for all uses
		while (pattern) {
			for ( var i = 0; i < DateTimeFormat.TOKENS_.length; ++i) {
				var m = pattern.match(DateTimeFormat.TOKENS_[i]);
				if (m) {
					var part = m[0];
					pattern = pattern.substring(part.length);
					if (i == DateTimeFormat.PartTypes_.QUOTED_STRING) {
						if (part == "''") {
							part = "'"; // '' -> '
						} else {
							part = part.substring(1, part.length - 1); // strip
																		// quotes
							part = part.replace(/\'\'/, "'");
						}
					}
					this.patternParts_.push( {
						text : part,
						type : i
					});
					break;
				}
			}
		}
	};

	/**
	 * Format the given date object according to preset pattern and current
	 * locale.
	 * 
	 * @param {Date}
	 *            date The Date object that is being formatted.
	 * @param {object} Options for formatting :
	 * 		- timeZone {jef.i18n.TimeZone} optional, if specified, time related fields will
	 *            be formatted based on its setting. When this field is not
	 *            specified, "undefined" will be pass around and those function
	 *            that really need time zone service will create a default one.
	 * @return {string} Formatted string for the given date.
	 */
	DateTimeFormat.prototype.format = function(date, options){
		// We don't want to write code to calculate each date field because we
		// want to maximize performance and minimize code size.
		// JavaScript only provide API to render local time.
		// Suppose target date is: 16:00 GMT-0400
		// OS local time is: 12:00 GMT-0800
		// We want to create a Local Date Object : 16:00 GMT-0800, and fix the
		// time zone display ourselves.
		// Thing get a little bit tricky when daylight time transition happens.
		// For
		// example, suppose OS timeZone is America/Los_Angeles, it is impossible
		// to
		// represent "2006/4/2 02:30" even for those timeZone that has no
		// transition
		// at this time. Because 2:00 to 3:00 on that day does not exising in
		// America/Los_Angeles time zone. To avoid calculating date field
		// through
		// our own code, we uses 3 Date object instead, one for "Year, month,
		// day",
		// one for time within that day, and one for timeZone object since it
		// need
		// the real time to figure out actual time zone offset.
		options = options || {};
		opt_timeZone = options.timeZone || this.default_timeZone;		
		var diff = opt_timeZone ? (date.getTimezoneOffset() - opt_timeZone
				.getOffset(date)) * 60000 : 0;
		var dateForDate = diff ? new Date(date.getTime() + diff) : date;
		var dateForTime = dateForDate;
		// in daylight time switch on/off hour, diff adjustment could alter time
		// because of timeZone offset change, move 1 day forward or backward.
		if (opt_timeZone
				&& dateForDate.getTimezoneOffset() != date.getTimezoneOffset()) {
			diff += diff > 0 ? -24 * 60 * 60000 : 24 * 60 * 60000;
			dateForTime = new Date(date.getTime() + diff);
		}

		var out = [];
		for ( var i = 0; i < this.patternParts_.length; ++i) {
			var text = this.patternParts_[i].text;
			if (DateTimeFormat.PartTypes_.FIELD == this.patternParts_[i].type) {
				out.push(this.formatField_(text, date, dateForDate,
						dateForTime, opt_timeZone));
			} else {
				out.push(text);
			}
		}
		return out.join('');
	};

	/**
	 * Apply a predefined pattern as identified by formatType, which is stored
	 * in locale specific repository.
	 * 
	 * @param {number}
	 *            formatType A number that identified the predefined pattern.
	 * @private
	 */
	DateTimeFormat.prototype.applyStandardPattern_ = function(
			formatType) {
		var pattern;
		if (formatType < 4) {
			pattern = this.dateTimeSymbols.DATEFORMATS[formatType];
		} else if (formatType < 8) {
			pattern = this.dateTimeSymbols.TIMEFORMATS[formatType - 4];
		} else if (formatType < 12) {
			pattern = this.dateTimeSymbols.DATEFORMATS[formatType - 8] + ' '
					+ this.dateTimeSymbols.TIMEFORMATS[formatType - 8];
		} else {
			this
					.applyStandardPattern_(DateTimeFormat.Format.MEDIUM_DATETIME);
			return;
		}
		this.applyPattern_(pattern);
	};

	/**
	 * Formats Era field according to pattern specified.
	 * 
	 * @param {number}
	 *            count Number of time pattern char repeats, it controls how a
	 *            field should be formatted.
	 * @param {Date}
	 *            date It holds the date object to be formatted.
	 * @return {string} Formatted string that represent this field.
	 * @private
	 */
	DateTimeFormat.prototype.formatEra_ = function(count, date) {
		var value = date.getFullYear() > 0 ? 1 : 0;
		return count >= 4 ? this.dateTimeSymbols.ERANAMES[value]
				: this.dateTimeSymbols.ERAS[value];
	};

	/**
	 * Formats Year field according to pattern specified Javascript Date object
	 * seems incapable handling 1BC and year before. It can show you year 0
	 * which does not exists. following we just keep consistent with
	 * javascript's toString method. But keep in mind those things should be
	 * unsupported.
	 * 
	 * @param {number}
	 *            count Number of time pattern char repeats, it controls how a
	 *            field should be formatted.
	 * @param {Date}
	 *            date It holds the date object to be formatted.
	 * @return {string} Formatted string that represent this field.
	 * @private
	 */
	DateTimeFormat.prototype.formatYear_ = function(count, date) {
		var value = date.getFullYear();
		if (value < 0) {
			value = -value;
		}
		return count == 2 ? string.padNumber(value % 100, 2)
				: String(value);
	};

	/**
	 * Formats Month field according to pattern specified
	 * 
	 * @param {number}
	 *            count Number of time pattern char repeats, it controls how a
	 *            field should be formatted.
	 * @param {Date}
	 *            date It holds the date object to be formatted.
	 * @return {string} Formatted string that represent this field.
	 * @private
	 */
	DateTimeFormat.prototype.formatMonth_ = function(count, date) {
		var value = date.getMonth();
		switch (count) {
		case 5:
			return this.dateTimeSymbols.NARROWMONTHS[value];
		case 4:
			return this.dateTimeSymbols.MONTHS[value];
		case 3:
			return this.dateTimeSymbols.SHORTMONTHS[value];
		default:
			return string.padNumber(value + 1, count);
		}
	};

	/**
	 * Formats (1..24) Hours field according to pattern specified
	 * 
	 * @param {number}
	 *            count Number of time pattern char repeats. This controls how a
	 *            field should be formatted.
	 * @param {Date}
	 *            date It holds the date object to be formatted.
	 * @return {string} Formatted string that represent this field.
	 * @private
	 */
	DateTimeFormat.prototype.format24Hours_ = function(count, date) {
		return string.padNumber(date.getHours() || 24, count);
	};

	/**
	 * Formats Fractional seconds field according to pattern specified
	 * 
	 * @param {number}
	 *            count Number of time pattern char repeats, it controls how a
	 *            field should be formatted.
	 * @param {Date}
	 *            date It holds the date object to be formatted.
	 * 
	 * @return {string} Formatted string that represent this field.
	 * @private
	 */
	DateTimeFormat.prototype.formatFractionalSeconds_ = function(
			count, date) {
		// Fractional seconds left-justify, append 0 for precision beyond 3
		var value = date.getTime() % 1000 / 1000;
		return value.toFixed(Math.min(3, count)).substr(2)
				+ (count > 3 ? string.padNumber(0, count - 3) : '');
	};

	/**
	 * Formats Day of week field according to pattern specified
	 * 
	 * @param {number}
	 *            count Number of time pattern char repeats, it controls how a
	 *            field should be formatted.
	 * @param {Date}
	 *            date It holds the date object to be formatted.
	 * @return {string} Formatted string that represent this field.
	 * @private
	 */
	DateTimeFormat.prototype.formatDayOfWeek_ = function(count, date) {
		var value = date.getDay();
		return count >= 4 ? this.dateTimeSymbols.WEEKDAYS[value]
				: this.dateTimeSymbols.SHORTWEEKDAYS[value];
	};

	/**
	 * Formats Am/Pm field according to pattern specified
	 * 
	 * @param {number}
	 *            count Number of time pattern char repeats, it controls how a
	 *            field should be formatted.
	 * @param {Date}
	 *            date It holds the date object to be formatted.
	 * @return {string} Formatted string that represent this field.
	 * @private
	 */
	DateTimeFormat.prototype.formatAmPm_ = function(count, date) {
		var hours = date.getHours();
		return this.dateTimeSymbols.AMPMS[hours >= 12 && hours < 24 ? 1 : 0];
	};

	/**
	 * Formats (1..12) Hours field according to pattern specified
	 * 
	 * @param {number}
	 *            count Number of time pattern char repeats, it controls how a
	 *            field should be formatted.
	 * @param {Date}
	 *            date It holds the date object to be formatted.
	 * @return {string} formatted string that represent this field.
	 * @private
	 */
	DateTimeFormat.prototype.format1To12Hours_ = function(count,
			date) {
		return string.padNumber(date.getHours() % 12 || 12, count);
	};

	/**
	 * Formats (0..11) Hours field according to pattern specified
	 * 
	 * @param {number}
	 *            count Number of time pattern char repeats, it controls how a
	 *            field should be formatted.
	 * @param {Date}
	 *            date It holds the date object to be formatted.
	 * @return {string} formatted string that represent this field.
	 * @private
	 */
	DateTimeFormat.prototype.format0To11Hours_ = function(count,
			date) {
		return string.padNumber(date.getHours() % 12, count);
	};

	/**
	 * Formats (0..23) Hours field according to pattern specified
	 * 
	 * @param {number}
	 *            count Number of time pattern char repeats, it controls how a
	 *            field should be formatted.
	 * @param {Date}
	 *            date It holds the date object to be formatted.
	 * @return {string} formatted string that represent this field.
	 * @private
	 */
	DateTimeFormat.prototype.format0To23Hours_ = function(count,
			date) {
		return string.padNumber(date.getHours(), count);
	};

	/**
	 * Formats Standalone weekday field according to pattern specified
	 * 
	 * @param {number}
	 *            count Number of time pattern char repeats, it controls how a
	 *            field should be formatted.
	 * @param {Date}
	 *            date It holds the date object to be formatted.
	 * @return {string} formatted string that represent this field.
	 * @private
	 */
	DateTimeFormat.prototype.formatStandaloneDay_ = function(count,
			date) {
		var value = date.getDay();
		switch (count) {
		case 5:
			return this.dateTimeSymbols.STANDALONENARROWWEEKDAYS[value];
		case 4:
			return this.dateTimeSymbols.STANDALONEWEEKDAYS[value];
		case 3:
			return this.dateTimeSymbols.STANDALONESHORTWEEKDAYS[value];
		default:
			return string.padNumber(value, 1);
		}
	};

	/**
	 * Formats Standalone Month field according to pattern specified
	 * 
	 * @param {number}
	 *            count Number of time pattern char repeats, it controls how a
	 *            field should be formatted.
	 * @param {Date}
	 *            date It holds the date object to be formatted.
	 * @return {string} formatted string that represent this field.
	 * @private
	 */
	DateTimeFormat.prototype.formatStandaloneMonth_ = function(
			count, date) {
		var value = date.getMonth();
		switch (count) {
		case 5:
			return this.dateTimeSymbols.STANDALONENARROWMONTHS[value];
		case 4:
			return this.dateTimeSymbols.STANDALONEMONTHS[value];
		case 3:
			return this.dateTimeSymbols.STANDALONESHORTMONTHS[value];
		default:
			return string.padNumber(value + 1, count);
		}
	};

	/**
	 * Formats Quarter field according to pattern specified
	 * 
	 * @param {number}
	 *            count Number of time pattern char repeats, it controls how a
	 *            field should be formatted.
	 * @param {Date}
	 *            date It holds the date object to be formatted.
	 * @return {string} Formatted string that represent this field.
	 * @private
	 */
	DateTimeFormat.prototype.formatQuarter_ = function(count, date) {
		var value = Math.floor(date.getMonth() / 3);
		return count < 4 ? this.dateTimeSymbols.SHORTQUARTERS[value]
				: this.dateTimeSymbols.QUARTERS[value];
	};

	/**
	 * Formats Date field according to pattern specified
	 * 
	 * @param {number}
	 *            count Number of time pattern char repeats, it controls how a
	 *            field should be formatted.
	 * @param {Date}
	 *            date It holds the date object to be formatted.
	 * @return {string} Formatted string that represent this field.
	 * @private
	 */
	DateTimeFormat.prototype.formatDate_ = function(count, date) {
		return string.padNumber(date.getDate(), count);
	};

	/**
	 * Formats Minutes field according to pattern specified
	 * 
	 * @param {number}
	 *            count Number of time pattern char repeats, it controls how a
	 *            field should be formatted.
	 * @param {Date}
	 *            date It holds the date object to be formatted.
	 * @return {string} Formatted string that represent this field.
	 * @private
	 */
	DateTimeFormat.prototype.formatMinutes_ = function(count, date) {
		return string.padNumber(date.getMinutes(), count);
	};

	/**
	 * Formats Seconds field according to pattern specified
	 * 
	 * @param {number}
	 *            count Number of time pattern char repeats, it controls how a
	 *            field should be formatted.
	 * @param {Date}
	 *            date It holds the date object to be formatted.
	 * @return {string} Formatted string that represent this field.
	 * @private
	 */
	DateTimeFormat.prototype.formatSeconds_ = function(count, date) {
		return string.padNumber(date.getSeconds(), count);
	};

	/**
	 * Formats TimeZone field following RFC
	 * 
	 * @param {number}
	 *            count Number of time pattern char repeats, it controls how a
	 *            field should be formatted.
	 * @param {Date}
	 *            date It holds the date object to be formatted.
	 * @param {jef.i18n.TimeZone}
	 *            opt_timeZone This holds current time zone info.
	 * @return {string} Formatted string that represent this field.
	 * @private
	 */
	DateTimeFormat.prototype.formatTimeZoneRFC_ = function(count,
			date, opt_timeZone) {
		opt_timeZone = opt_timeZone
				|| i18n.TimeZone.createTimeZone(date.getTimezoneOffset());
		return count < 4 ? opt_timeZone.getRFCTimeZoneString(date)
				: opt_timeZone.getGMTString(date);
	};

	/**
	 * Generate GMT timeZone string for given date
	 * 
	 * @param {number}
	 *            count Number of time pattern char repeats, it controls how a
	 *            field should be formatted.
	 * @param {Date}
	 *            date Whose value being evaluated.
	 * @param {jef.i18n.TimeZone}
	 *            opt_timeZone This holds current time zone info.
	 * @return {string} GMT timeZone string.
	 * @private
	 */
	DateTimeFormat.prototype.formatTimeZone_ = function(count, date,
			opt_timeZone) {
		opt_timeZone = opt_timeZone
				|| i18n.TimeZone.createTimeZone(date.getTimezoneOffset());
		return count < 4 ? opt_timeZone.getShortName(date) : opt_timeZone
				.getLongName(date);
	};

	/**
	 * Generate GMT timeZone string for given date
	 * 
	 * @param {Date}
	 *            date Whose value being evaluated.
	 * @param {jef.i18n.TimeZone}
	 *            opt_timeZone This holds current time zone info.
	 * @return {string} GMT timeZone string.
	 * @private
	 */
	DateTimeFormat.prototype.formatTimeZoneId_ = function(date,
			opt_timeZone) {
		opt_timeZone = opt_timeZone
				|| i18n.TimeZone.createTimeZone(date.getTimezoneOffset());
		return opt_timeZone.getTimeZoneId();
	};

	/**
	 * Formatting one date field.
	 * 
	 * @param {string}
	 *            patternStr The pattern string for the field being formatted.
	 * @param {Date}
	 *            date represents the real date to be formatted.
	 * @param {Date}
	 *            dateForDate used to resolve date fields for formatting.
	 * @param {Date}
	 *            dateForTime used to resolve time fields for formatting.
	 * @param {jef.i18n.TimeZone}
	 *            opt_timeZone This holds current time zone info.
	 * @return {string} string representation for the given field.
	 * @private
	 */
	DateTimeFormat.prototype.formatField_ = function(patternStr,
			date, dateForDate, dateForTime, opt_timeZone) {
		var count = patternStr.length;
		switch (patternStr.charAt(0)) {
		case 'G':
			return this.formatEra_(count, dateForDate);
		case 'y':
			return this.formatYear_(count, dateForDate);
		case 'M':
			return this.formatMonth_(count, dateForDate);
		case 'k':
			return this.format24Hours_(count, dateForTime);
		case 'S':
			return this.formatFractionalSeconds_(count, dateForTime);
		case 'E':
			return this.formatDayOfWeek_(count, dateForDate);
		case 'a':
			return this.formatAmPm_(count, dateForTime);
		case 'h':
			return this.format1To12Hours_(count, dateForTime);
		case 'K':
			return this.format0To11Hours_(count, dateForTime);
		case 'H':
			return this.format0To23Hours_(count, dateForTime);
		case 'c':
			return this.formatStandaloneDay_(count, dateForDate);
		case 'L':
			return this.formatStandaloneMonth_(count, dateForDate);
		case 'Q':
			return this.formatQuarter_(count, dateForDate);
		case 'd':
			return this.formatDate_(count, dateForDate);
		case 'm':
			return this.formatMinutes_(count, dateForTime);
		case 's':
			return this.formatSeconds_(count, dateForTime);
		case 'v':
			return this.formatTimeZoneId_(date, opt_timeZone);
		case 'z':
			return this.formatTimeZone_(count, date, opt_timeZone);
		case 'Z':
			return this.formatTimeZoneRFC_(count, date, opt_timeZone);
		default:
			return '';
		}
	};
	
	jef.date.JavaDateTimeFormat = DateTimeFormat;

})(jef, jef.i18n, jef.string);

//
// Date Time Parse
//

(function(jef, i18n) {

	var DateTimeFormat = jef.date.JavaDateTimeFormat;
	
	if (jef.isDef(jef.date.JavaDateTimeParse))
		return;

	/**
	 * DateTimeParse is for parsing date in a locale-sensitive manner. It allows
	 * user to use any customized patterns to parse date-time string under
	 * certain locale. Things varies across locales like month name, weekname,
	 * field order, etc.
	 * 
	 * This module is the counter-part of DateTimeFormat. They use the same
	 * date/time pattern specification, which is borrowed from ICU/JDK.
	 * 
	 * This implementation could parse partial date/time.
	 * 
	 * Time Format Syntax: To specify the time format use a time pattern string.
	 * In this pattern, following letters are reserved as pattern letters, which
	 * are defined as the following:
	 * 
	 * <pre>
	 * Symbol   Meaning                 Presentation        Example
	 * ------   -------                 ------------        -------
	 * G        era designator          (Text)              AD
	 * y#       year                    (Number)            1996
	 * M        month in year           (Text &amp; Number)     July &amp; 07
	 * d        day in month            (Number)            10
	 * h        hour in am/pm (1&tilde;12)    (Number)            12
	 * H        hour in day (0&tilde;23)      (Number)            0
	 * m        minute in hour          (Number)            30
	 * s        second in minute        (Number)            55
	 * S        fractional second       (Number)            978
	 * E        day of week             (Text)              Tuesday
	 * D        day in year             (Number)            189
	 * a        am/pm marker            (Text)              PM
	 * k        hour in day (1&tilde;24)      (Number)            24
	 * K        hour in am/pm (0&tilde;11)    (Number)            0
	 * z        time zone               (Text)              Pacific Standard Time
	 * Z        time zone (RFC 822)     (Number)            -0800
	 * v        time zone (generic)     (Text)              Pacific Time
	 * '        escape for text         (Delimiter)         'Date='
	 * ''       single quote            (Literal)           'o''clock'
	 * </pre>
	 * 
	 * The count of pattern letters determine the format.
	 * <p>
	 * (Text): 4 or more pattern letters--use full form, less than 4--use short
	 * or abbreviated form if one exists. In parsing, we will always try long
	 * format, then short.
	 * <p>
	 * (Number): the minimum number of digits.
	 * <p>
	 * (Text & Number): 3 or over, use text, otherwise use number.
	 * <p>
	 * Any characters that not in the pattern will be treated as quoted text.
	 * For instance, characters like ':', '.', ' ', '#' and '@' will appear in
	 * the resulting time text even they are not embraced within single quotes.
	 * In our current pattern usage, we didn't use up all letters. But those
	 * unused letters are strongly discouraged to be used as quoted text without
	 * quote. That's because we may use other letter for pattern in future.
	 * <p>
	 * 
	 * Examples Using the US Locale:
	 * 
	 * Format Pattern Result -------------- ------- "yyyy.MM.dd G 'at' HH:mm:ss
	 * vvvv" ->> 1996.07.10 AD at 15:08:56 Pacific Time "EEE, MMM d, ''yy" ->>
	 * Wed, July 10, '96 "h:mm a" ->> 12:08 PM "hh 'o''clock' a, zzzz" ->> 12
	 * o'clock PM, Pacific Daylight Time "K:mm a, vvv" ->> 0:00 PM, PT
	 * "yyyyy.MMMMM.dd GGG hh:mm aaa" ->> 01996.July.10 AD 12:08 PM
	 * 
	 * <p>
	 * When parsing a date string using the abbreviated year pattern ("yy"),
	 * DateTimeParse must interpret the abbreviated year relative to some
	 * century. It does this by adjusting dates to be within 80 years before and
	 * 20 years after the time the parse function is called. For example, using
	 * a pattern of "MM/dd/yy" and a DateTimeParse instance created on Jan 1,
	 * 1997, the string "01/11/12" would be interpreted as Jan 11, 2012 while
	 * the string "05/04/64" would be interpreted as May 4, 1964. During
	 * parsing, only strings consisting of exactly two digits, as defined by
	 * {@link java.lang.Character#isDigit(char)}, will be parsed into the
	 * default century. Any other numeric string, such as a one digit string, a
	 * three or more digit string will be interpreted as its face value.
	 * 
	 * <p>
	 * If the year pattern does not have exactly two 'y' characters, the year is
	 * interpreted literally, regardless of the number of digits. So using the
	 * pattern "MM/dd/yyyy", "01/11/12" parses to Jan 11, 12 A.D.
	 * 
	 * <p>
	 * When numeric fields abut one another directly, with no intervening
	 * delimiter characters, they constitute a run of abutting numeric fields.
	 * Such runs are parsed specially. For example, the format "HHmmss" parses
	 * the input text "123456" to 12:34:56, parses the input text "12345" to
	 * 1:23:45, and fails to parse "1234". In other words, the leftmost field of
	 * the run is flexible, while the others keep a fixed width. If the parse
	 * fails anywhere in the run, then the leftmost field is shortened by one
	 * character, and the entire run is parsed again. This is repeated until
	 * either the parse succeeds or the leftmost field is one character in
	 * length. If the parse still fails at that point, the parse of the run
	 * fails.
	 * 
	 * <p>
	 * Now timezone parsing only support GMT:hhmm, GMT:+hhmm, GMT:-hhmm
	 * 
	 */

	/**
	 * Construct a DateTimeParse based on current locale.
	 * 
	 * @param {string|number} pattern Pattern specification or pattern type.
	 * @param {object} Options : 
	 * 	- locale {string} Locale used for this parse process
	 * @constructor
	 */
	var DateTimeParse = function(pattern, options) {
		options = options || {};
		this.patternParts_ = [];
		this.dateTimeSymbols = i18n.DateTimeSymbols;
		if (jef.isDefAndNotNull(options.locale)) {
			var symbolsName = 'DateTimeSymbols_' + options.locale;
			var symbols = i18n[symbolsName];
			jef.assert(jef.isDef(symbols),
					"Date time symbols aren't defined for locale : "
							+ options.locale);
			this.dateTimeSymbols = symbols;
		}
		
		if (typeof pattern == 'number') {
			this.applyStandardPattern_(pattern);
		} else {
			this.applyPattern_(pattern);
		}
	};

	/**
	 * Number of years prior to now that the century used to disambiguate two
	 * digit years will begin
	 * 
	 * @type {number}
	 */
	DateTimeParse.ambiguousYearCenturyStart = 80;
	
	/**
	* Min valid year.
	*/
	DateTimeParse.minValidYear = 100;
	/**
	* Max valid year
	*/
	DateTimeParse.maxValidYear = 9999;

	/**
	 * Apply a pattern to this Parser. The pattern string will be parsed and
	 * saved in "compiled" form. Note: this method is somewhat similar to the
	 * pattern parsing methold in datetimeformat. If you see something wrong
	 * here, you might want to check the other.
	 * 
	 * @param {string}
	 *            pattern It describes the format of date string that need to be
	 *            parsed.
	 * @private
	 */
	DateTimeParse.prototype.applyPattern_ = function(pattern) {
		var inQuote = false;
		var buf = '';

		for ( var i = 0; i < pattern.length; i++) {
			var ch = pattern.charAt(i);

			// handle space, add literal part (if exist), and add space part
			if (ch == ' ') {
				if (buf.length > 0) {
					this.patternParts_.push( {
						text : buf,
						count : 0,
						abutStart : false
					});
					buf = '';
				}
				this.patternParts_.push( {
					text : ' ',
					count : 0,
					abutStart : false
				});
				while (i < pattern.length - 1 && pattern.charAt(i + 1) == ' ') {
					i++;
				}
			} else if (inQuote) {
				// inside quote, except '', just copy or exit
				if (ch == '\'') {
					if (i + 1 < pattern.length && pattern.charAt(i + 1) == '\'') {
						// quote appeared twice continuously, interpret as one
						// quote.
						buf += '\'';
						i++;
					} else {
						// exit quote
						inQuote = false;
					}
				} else {
					// literal
					buf += ch;
				}
			} else if (DateTimeParse.PATTERN_CHARS_.indexOf(ch) >= 0) {
				// outside quote, it is a pattern char
				if (buf.length > 0) {
					this.patternParts_.push( {
						text : buf,
						count : 0,
						abutStart : false
					});
					buf = '';
				}
				var count = this.getNextCharCount_(pattern, i);
				this.patternParts_.push( {
					text : ch,
					count : count,
					abutStart : false
				});
				i += count - 1;
			} else if (ch == '\'') {
				// Two consecutive quotes is a quote literal, inside or outside
				// of quotes.
				if (i + 1 < pattern.length && pattern.charAt(i + 1) == '\'') {
					buf += '\'';
					i++;
				} else {
					inQuote = true;
				}
			} else {
				buf += ch;
			}
		}

		if (buf.length > 0) {
			this.patternParts_.push( {
				text : buf,
				count : 0,
				abutStart : false
			});
		}

		this.markAbutStart_();
	};

	/**
	 * Apply a predefined pattern to this Parser.
	 * 
	 * @param {number}
	 *            formatType A constant used to identified the predefined
	 *            pattern string stored in locale repository.
	 * @private
	 */
	DateTimeParse.prototype.applyStandardPattern_ = function(
			formatType) {
		var pattern;
		// formatType constants are in consecutive numbers. So it can be used to
		// index array in following way.

		// if type is out of range, default to medium date/time format.
		if (formatType > DateTimeFormat.Format.SHORT_DATETIME) {
			formatType = DateTimeFormat.Format.MEDIUM_DATETIME;
		}

		if (formatType < 4) {
			pattern = this.dateTimeSymbols.DATEFORMATS[formatType];
		} else if (formatType < 8) {
			pattern = this.dateTimeSymbols.TIMEFORMATS[formatType - 4];
		} else {
			pattern = this.dateTimeSymbols.DATEFORMATS[formatType - 8] + ' '
					+ this.dateTimeSymbols.TIMEFORMATS[formatType - 8];
		}
		this.applyPattern_(pattern);
	};

	/**
	 * Parse the given string and fill info into date object. This version does
	 * not validate the input.
	 * 
	 * @param {string} text The string being parsed.
	 * @param {object} options Options for parsing : 
	 * 		- date {Date} The Date object to hold the parsed date.   
	 * 		- pos {Array.<number>=} Position to pass in and get back. It not given, 
	 * 			process will assume that text end must be reach at the end of parse process
	 * 		- valid {boolean} If process must valid date (ie : date 30/02/2010 is not valid) 
	 * @return {Date} parsed date or null (if error).
	 */
	DateTimeParse.prototype.parse = function(text, options) { //opt_date, opt_pos, opt_valid) {
		options = options || {};
		var mustReachEnd = options.pos ? false : true;
		options.valid = options.valid === undefined ? true : !!options.valid;
		options.date = options.date || new Date();
		options.pos = options.pos || [0];
		var advanced = this.internalParse_(text, options.date, options.pos[0], options.valid);
		options.pos[0] += advanced;
		var hasReachEnd = text.length == advanced;
		if(mustReachEnd) { delete options.pos; }
		return advanced != 0 && (!mustReachEnd || hasReachEnd) ? options.date : null;
	};


	/**
	 * Parse the given string and fill info into date object.
	 * 
	 * @param {string}
	 *            text The string being parsed.
	 * @param {Date}
	 *            date The Date object to hold the parsed date.
	 * @param {number}
	 *            start The position from where parse should begin.
	 * @param {boolean}
	 *            validation If true, input string need to be a valid date/time
	 *            string.
	 * @return {number} How many characters parser advanced.
	 * @private
	 */
	DateTimeParse.prototype.internalParse_ = function(text, date,
			start, validation) {
		var cal = new DateTimeParse.MyDate_();
		var parsePos = [ start ];

		// For parsing abutting numeric fields. 'abutPat' is the
		// offset into 'pattern' of the first of 2 or more abutting
		// numeric fields. 'abutStart' is the offset into 'text'
		// where parsing the fields begins. 'abutPass' starts off as 0
		// and increments each time we try to parse the fields.
		var abutPat = -1; // If >=0, we are in a run of abutting numeric
							// fields
		var abutStart = 0;
		var abutPass = 0;

		for ( var i = 0; i < this.patternParts_.length; i++) {
			if (this.patternParts_[i].count > 0) {
				if (abutPat < 0 && this.patternParts_[i].abutStart) {
					abutPat = i;
					abutStart = start;
					abutPass = 0;
				}

				// Handle fields within a run of abutting numeric fields. Take
				// the pattern "HHmmss" as an example. We will try to parse
				// 2/2/2 characters of the input text, then if that fails,
				// 1/2/2. We only adjust the width of the leftmost field; the
				// others remain fixed. This allows "123456" => 12:34:56, but
				// "12345" => 1:23:45. Likewise, for the pattern "yyyyMMdd" we
				// try 4/2/2, 3/2/2, 2/2/2, and finally 1/2/2.
				if (abutPat >= 0) {
					// If we are at the start of a run of abutting fields, then
					// shorten this field in each pass. If we can't shorten
					// this field any more, then the parse of this set of
					// abutting numeric fields has failed.
					var count = this.patternParts_[i].count;
					if (i == abutPat) {
						count -= abutPass;
						abutPass++;
						if (count == 0) {
							// tried all possible width, fail now
							return 0;
						}
					}

					if (!this.subParse_(text, parsePos, this.patternParts_[i],
							count, cal)) {
						// If the parse fails anywhere in the run, back up to
						// the
						// start of the run and retry.
						i = abutPat - 1;
						parsePos[0] = abutStart;
						continue;
					}
				}

				// Handle non-numeric fields and non-abutting numeric fields.
				else {
					abutPat = -1;
					if (!this.subParse_(text, parsePos, this.patternParts_[i],
							0, cal)) {
						return 0;
					}
				}
			} else {
				// Handle literal pattern characters. These are any
				// quoted characters and non-alphabetic unquoted
				// characters.
				abutPat = -1;
				// A run of white space in the pattern matches a run
				// of white space in the input text.
				if (this.patternParts_[i].text.charAt(0) == ' ') {
					// Advance over run in input text
					var s = parsePos[0];
					this.skipSpace_(text, parsePos);

					// Must see at least one white space char in input
					if (parsePos[0] > s) {
						continue;
					}
				} else if (text
						.indexOf(this.patternParts_[i].text, parsePos[0]) == parsePos[0]) {
					parsePos[0] += this.patternParts_[i].text.length;
					continue;
				}
				// We fall through to this point if the match fails
				return 0;
			}
		}

		// return progress
		return cal.calcDate_(date, validation) ? parsePos[0] - start : 0;
	};

	/**
	 * Calculate character repeat count in pattern.
	 * 
	 * @param {string}
	 *            pattern It describes the format of date string that need to be
	 *            parsed.
	 * @param {number}
	 *            start The position of pattern character.
	 * 
	 * @return {number} Repeat count.
	 * @private
	 */
	DateTimeParse.prototype.getNextCharCount_ = function(pattern,
			start) {
		var ch = pattern.charAt(start);
		var next = start + 1;
		while (next < pattern.length && pattern.charAt(next) == ch) {
			next++;
		}
		return next - start;
	};

	/**
	 * All acceptable pattern characters.
	 * 
	 * @private
	 */
	DateTimeParse.PATTERN_CHARS_ = 'GyMdkHmsSEDahKzZvQ';

	/**
	 * Pattern characters that specify numerical field.
	 * 
	 * @private
	 */
	DateTimeParse.NUMERIC_FORMAT_CHARS_ = 'MydhHmsSDkK';

	/**
	 * Check if the pattern part is a numeric field.
	 * 
	 * @param {Object}
	 *            part pattern part to be examined.
	 * 
	 * @return {boolean} true if the pattern part is numberic field.
	 * @private
	 */
	DateTimeParse.prototype.isNumericField_ = function(part) {
		if (part.count <= 0) {
			return false;
		}
		var i = DateTimeParse.NUMERIC_FORMAT_CHARS_
				.indexOf(part.text.charAt(0));
		return i > 0 || i == 0 && part.count < 3;
	};

	/**
	 * Identify the start of an abutting numeric fields' run. Taking pattern
	 * "HHmmss" as an example. It will try to parse 2/2/2 characters of the
	 * input text, then if that fails, 1/2/2. We only adjust the width of the
	 * leftmost field; the others remain fixed. This allows "123456" =>
	 * 12:34:56, but "12345" => 1:23:45. Likewise, for the pattern "yyyyMMdd" we
	 * try 4/2/2, 3/2/2, 2/2/2, and finally 1/2/2. The first field of connected
	 * numeric fields will be marked as abutStart, its width can be reduced to
	 * accomodate others.
	 * 
	 * @private
	 */
	DateTimeParse.prototype.markAbutStart_ = function() {
		// abut parts are continuous numeric parts. abutStart is the switch
		// point from non-abut to abut
		var abut = false;

		for ( var i = 0; i < this.patternParts_.length; i++) {
			if (this.isNumericField_(this.patternParts_[i])) {
				// if next part is not following abut sequence, and
				// isNumericField_
				if (!abut && i + 1 < this.patternParts_.length
						&& this.isNumericField_(this.patternParts_[i + 1])) {
					abut = true;
					this.patternParts_[i].abutStart = true;
				}
			} else {
				abut = false;
			}
		}
	};

	/**
	 * Skip space in the string.
	 * 
	 * @param {string}
	 *            text input string.
	 * @param {Array.
	 *            <number>} pos where skip start, and return back where the skip
	 *            stops.
	 * @private
	 */
	DateTimeParse.prototype.skipSpace_ = function(text, pos) {
		var m = text.substring(pos[0]).match(/^\s+/);
		if (m) {
			pos[0] += m[0].length;
		}
	};

	/**
	 * Protected method that converts one field of the input string into a
	 * numeric field value.
	 * 
	 * @param {string}
	 *            text the time text to be parsed.
	 * @param {Array.
	 *            <number>} pos Parse position.
	 * @param {Object}
	 *            part the pattern part for this field.
	 * @param {number}
	 *            digitCount when > 0, numeric parsing must obey the count.
	 * @param {DateTimeParse.MyDate_}
	 *            cal object that holds parsed value.
	 * 
	 * @return {boolean} True if it parses successfully.
	 * @private
	 */
	DateTimeParse.prototype.subParse_ = function(text, pos, part,
			digitCount, cal) {
		this.skipSpace_(text, pos);

		var start = pos[0];
		var ch = part.text.charAt(0);

		// parse integer value if it is a numeric field
		var value = -1;
		if (this.isNumericField_(part)) {
			if (digitCount > 0) {
				if ((start + digitCount) > text.length) {
					return false;
				}
				value = this.parseInt_(text.substring(0, start + digitCount),
						pos);
			} else {
				value = this.parseInt_(text, pos);
			}
			if(value == -1)
				return false;
		}

		switch (ch) {
		case 'G': // ERA
			cal.era = this.matchString_(text, pos, this.dateTimeSymbols.ERAS);
			return true;
		case 'M': // MONTH
			return this.subParseMonth_(text, pos, cal, value);
		case 'E':
			return this.subParseDayOfWeek_(text, pos, cal);
		case 'a': // AM_PM
			cal.ampm = this.matchString_(text, pos, this.dateTimeSymbols.AMPMS);
			return true;
		case 'y': // YEAR
			return this.subParseYear_(text, pos, start, value, part, cal);
		case 'Q': // QUARTER
			return this.subParseQuarter_(text, pos, cal, value);
		case 'd': // DATE
			cal.day = value;
			return true;
		case 'S': // FRACTIONAL_SECOND
			return this.subParseFractionalSeconds_(value, pos, start, cal);
		case 'h': // HOUR (1..12)
			if (value == 12) {
				value = 0;
			}
		case 'K': // HOUR (0..11)
		case 'H': // HOUR_OF_DAY (0..23)
		case 'k': // HOUR_OF_DAY (1..24)
			cal.hours = value;
			return true;
		case 'm': // MINUTE
			cal.minutes = value;
			return true;
		case 's': // SECOND
			cal.seconds = value;
			return true;

		case 'z': // ZONE_OFFSET
		case 'Z': // TIMEZONE_RFC
		case 'v': // TIMEZONE_GENERIC
			return this.subparseTimeZoneInGMT_(text, pos, cal);
		default:
			return false;
		}
	};

	/**
	 * Parse year field. Year field is special because 1) two digit year need to
	 * be resolved. 2) we allow year to take a sign. 3) year field participate
	 * in abut processing.
	 * 
	 * @param {string}
	 *            text the time text to be parsed.
	 * @param {Array.
	 *            <number>} pos Parse position.
	 * @param {number}
	 *            start where this field start.
	 * @param {number}
	 *            value integer value of year.
	 * @param {Object}
	 *            part the pattern part for this field.
	 * @param {DateTimeParse.MyDate_}
	 *            cal object to hold parsed value.
	 * 
	 * @return {boolean} True if successful.
	 * @private
	 */
	DateTimeParse.prototype.subParseYear_ = function(text, pos,
			start, value, part, cal) {
		var ch;
		if (value < 0) {
			// possible sign
			ch = text.charAt(pos[0]);
			if (ch != '+' && ch != '-') {
				return false;
			}
			pos[0]++;
			value = this.parseInt_(text, pos);
			if (value < 0) {
				return false;
			}
			if (ch == '-') {
				value = -value;
			}
		}

		// only if 2 digit was actually parsed, and pattern say it has 2 digit.
		if (!ch && pos[0] - start == 2 && part.count == 2) {
			cal.setTwoDigitYear_(value);
		} else {
			cal.year = value;
		}
		return true;
	};

	/**
	 * Parse Month field.
	 * 
	 * @param {string}
	 *            text the time text to be parsed.
	 * @param {Array.
	 *            <number>} pos Parse position.
	 * @param {DateTimeParse.MyDate_}
	 *            cal object to hold parsed value.
	 * @param {number}
	 *            value numeric value if this field is expressed using numeric
	 *            pattern, or -1 if not.
	 * 
	 * @return {boolean} True if parsing successful.
	 * @private
	 */
	DateTimeParse.prototype.subParseMonth_ = function(text, pos,
			cal, value) {
		// when month is symbols, i.e., MMM or MMMM, value will be -1
		if (value < 0) {
			// Want to be able to parse both short and long forms.
			// Try count == 4 first:
			value = this.matchString_(text, pos, this.dateTimeSymbols.MONTHS);
			if (value < 0) { // count == 4 failed, now try count == 3
				value = this.matchString_(text, pos,
						this.dateTimeSymbols.SHORTMONTHS);
			}
			if (value < 0) {
				return false;
			}
			cal.month = value;
			return true;
		} else {
			cal.month = value - 1;
			return true;
		}
	};

	/**
	 * Parse Quarter field.
	 * 
	 * @param {string}
	 *            text the time text to be parsed.
	 * @param {Array.
	 *            <number>} pos Parse position.
	 * @param {DateTimeParse.MyDate_}
	 *            cal object to hold parsed value.
	 * @param {number}
	 *            value numeric value if this field is expressed using numeric
	 *            pattern, or -1 if not.
	 * 
	 * @return {boolean} True if parsing successful.
	 * @private
	 */
	DateTimeParse.prototype.subParseQuarter_ = function(text, pos,
			cal, value) {
		// value should be -1, since this is a non-numeric field.
		if (value < 0) {
			// Want to be able to parse both short and long forms.
			// Try count == 4 first:
			value = this.matchString_(text, pos, this.dateTimeSymbols.QUARTERS);
			if (value < 0) { // count == 4 failed, now try count == 3
				value = this.matchString_(text, pos,
						this.dateTimeSymbols.SHORTQUARTERS);
			}
			if (value < 0) {
				return false;
			}
			cal.month = value * 3; // First month of quarter.
			cal.day = 1;
			return true;
		}
		return false;
	};

	/**
	 * Parse Day of week field.
	 * 
	 * @param {string}
	 *            text the time text to be parsed.
	 * @param {Array.
	 *            <number>} pos Parse position.
	 * @param {DateTimeParse.MyDate_}
	 *            cal object to hold parsed value.
	 * 
	 * @return {boolean} True if successful.
	 * @private
	 */
	DateTimeParse.prototype.subParseDayOfWeek_ = function(text, pos,
			cal) {
		// Handle both short and long forms.
		// Try count == 4 (DDDD) first:
		var value = this.matchString_(text, pos, this.dateTimeSymbols.WEEKDAYS);
		if (value < 0) {
			value = this.matchString_(text, pos,
					this.dateTimeSymbols.SHORTWEEKDAYS);
		}
		if (value < 0) {
			return false;
		}
		cal.dayOfWeek = value;
		return true;
	};

	/**
	 * Parse fractional seconds field.
	 * 
	 * @param {number}
	 *            value parsed numberic value.
	 * @param {Array.
	 *            <number>} pos current parse position.
	 * @param {number}
	 *            start where this field start.
	 * @param {DateTimeParse.MyDate_}
	 *            cal object to hold parsed value.
	 * 
	 * @return {boolean} True if successful.
	 * @private
	 */
	DateTimeParse.prototype.subParseFractionalSeconds_ = function(
			value, pos, start, cal) {
		// Fractional seconds left-justify
		var len = pos[0] - start;
		cal.milliseconds = len < 3 ? value * Math.pow(10, 3 - len) : Math
				.round(value / Math.pow(10, len - 3));
		return true;
	};

	/**
	 * Parse GMT type timezone.
	 * 
	 * @param {string}
	 *            text the time text to be parsed.
	 * @param {Array.
	 *            <number>} pos Parse position.
	 * @param {DateTimeParse.MyDate_}
	 *            cal object to hold parsed value.
	 * 
	 * @return {boolean} True if successful.
	 * @private
	 */
	DateTimeParse.prototype.subparseTimeZoneInGMT_ = function(text,
			pos, cal) {
		// First try to parse generic forms such as GMT-07:00. Do this first
		// in case localized DateFormatZoneData contains the string "GMT"
		// for a zone; in that case, we don't want to match the first three
		// characters of GMT+/-HH:MM etc.

		// For time zones that have no known names, look for strings
		// of the form:
		// GMT[+-]hours:minutes or
		// GMT[+-]hhmm or
		// GMT.
		if (text.indexOf('GMT', pos[0]) == pos[0]) {
			pos[0] += 3; // 3 is the length of GMT
			return this.parseTimeZoneOffset_(text, pos, cal);
		}

		// The same for UTC
		if (text.indexOf('UTC', pos[0]) == pos[0]) {
			pos[0] += 3; // 3 is the length of UTC
			return this.parseTimeZoneOffset_(text, pos, cal);
		}

		// Look for time zone string
		if (this.subParseZoneString_(text, pos, cal) == true) {
			return true;
		}

		// As a last resort, look for numeric timezones of the form
		// [+-]hhmm as specified by RFC 822. This code is actually
		// a little more permissive than RFC 822. It will try to do
		// its best with numbers that aren't strictly 4 digits long.
		return this.parseTimeZoneOffset_(text, pos, cal);
	};

	/**
	 * Parse time zone string.
	 * 
	 * @param text
	 *            {string} text the time text to be parsed.
	 * @param pos
	 *            {Array.<number>} pos Parse position.
	 * @param cal
	 *            {DateTimeParse.MyDate_} cal object to hold parsed
	 *            value.
	 * @return {boolean} True if successful.
	 */
	DateTimeParse.prototype.subParseZoneString_ = function(text,
			pos, cal) {

		// TODO: check for named time zones by looking through the locale
		// data from the DateFormatZoneData strings. should parse both short and
		// long
		// forms.

		// If CLT, no offset
		if (text.indexOf('CLT', pos[0]) == pos[0]) {
			pos[0] += 3;
			return true;
		}
		
		// If CLST, no offset
		if (text.indexOf('CLST', pos[0]) == pos[0]) {
			pos[0] += 4;
			return true;
		}

		return false;
	};

	/**
	 * Parse time zone offset.
	 * 
	 * @param {string}
	 *            text the time text to be parsed.
	 * @param {Array.
	 *            <number>} pos Parse position.
	 * @param {DateTimeParse.MyDate_}
	 *            cal object to hold parsed value.
	 * 
	 * @return {boolean} True if successful.
	 * @private
	 */
	DateTimeParse.prototype.parseTimeZoneOffset_ = function(text,
			pos, cal) {
		if (pos[0] >= text.length) {
			cal.tzOffset = 0;
			return true;
		}

		var sign = 1;
		switch (text.charAt(pos[0])) {
		case '-':
			sign = -1; // fall through
		case '+':
			pos[0]++;
		}

		// Look for hours:minutes or hhmm.
		var st = pos[0];
		var value = this.parseInt_(text, pos);
		if (value == 0 && pos[0] == st) {
			return false;
		}

		var offset;
		if (pos[0] < text.length && text.charAt(pos[0]) == ':') {
			// This is the hours:minutes case
			offset = value * 60;
			pos[0]++;
			st = pos[0];
			value = this.parseInt_(text, pos);
			if (value == 0 && pos[0] == st) {
				return false;
			}
			offset += value;
		} else {
			// This is the hhmm case.
			offset = value;
			// Assume "-23".."+23" refers to hours.
			if (offset < 24 && (pos[0] - st) <= 2) {
				offset *= 60;
			} else {
				// todo: this looks questionable, should have more error
				// checking
				offset = offset % 100 + offset / 100 * 60;
			}
		}

		offset *= sign;
		cal.tzOffset = -offset;
		return true;
	};

	/**
	 * Parse a integer string and return integer value.
	 * 
	 * @param {string}
	 *            text string being parsed.
	 * @param {Array.
	 *            <number>} pos parse position.
	 * 
	 * @return {number} Converted integer value.
	 * @private
	 */
	DateTimeParse.prototype.parseInt_ = function(text, pos) {
		var m = text.substring(pos[0]).match(/^\d+/);
		if (!m) {
			return -1;
		}
		pos[0] += m[0].length;
		return parseInt(m[0], 10);
	};

	/**
	 * Attempt to match the text at a given position against an array of
	 * strings. Since multiple strings in the array may match (for example, if
	 * the array contains "a", "ab", and "abc", all will match the input string
	 * "abcd") the longest match is returned.
	 * 
	 * @param {string}
	 *            text The string to match to.
	 * @param {Array.
	 *            <number>} pos parsing position.
	 * @param {Array.
	 *            <string>} data The string array of matching patterns.
	 * 
	 * @return {number} the new start position if matching succeeded; a negative
	 *         number indicating matching failure.
	 * @private
	 */
	DateTimeParse.prototype.matchString_ = function(text, pos, data) {
		// There may be multiple strings in the data[] array which begin with
		// the same prefix (e.g., Cerven and Cervenec (June and July) in Czech).
		// We keep track of the longest match, and return that. Note that this
		// unfortunately requires us to test all array elements.
		var bestMatchLength = 0;
		var bestMatch = -1;
		var lower_text = text.substring(pos[0]).toLowerCase();
		for ( var i = 0; i < data.length; i++) {
			var len = data[i].length;
			// Always compare if we have no match yet; otherwise only compare
			// against potentially better matches (longer strings).
			if (len > bestMatchLength
					&& lower_text.indexOf(data[i].toLowerCase()) == 0) {
				bestMatch = i;
				bestMatchLength = len;
			}
		}
		if (bestMatch >= 0) {
			pos[0] += bestMatchLength;
		}
		return bestMatch;
	};

	/**
	 * This class hold the intermediate parsing result. After all fields are
	 * consumed, final result will be resolved from this class.
	 * 
	 * @constructor
	 * @private
	 */
	DateTimeParse.MyDate_ = function() {
	};

	/**
	 * The date's era.
	 * 
	 * @type {number?}
	 */
	DateTimeParse.MyDate_.prototype.era;

	/**
	 * The date's year.
	 * 
	 * @type {number?}
	 */
	DateTimeParse.MyDate_.prototype.year;

	/**
	 * The date's month.
	 * 
	 * @type {number?}
	 */
	DateTimeParse.MyDate_.prototype.month;

	/**
	 * The date's day of month.
	 * 
	 * @type {number?}
	 */
	DateTimeParse.MyDate_.prototype.day;

	/**
	 * The date's hour.
	 * 
	 * @type {number?}
	 */
	DateTimeParse.MyDate_.prototype.hours;

	/**
	 * The date's before/afternoon denominator.
	 * 
	 * @type {number?}
	 */
	DateTimeParse.MyDate_.prototype.ampm;

	/**
	 * The date's minutes.
	 * 
	 * @type {number?}
	 */
	DateTimeParse.MyDate_.prototype.minutes;

	/**
	 * The date's seconds.
	 * 
	 * @type {number?}
	 */
	DateTimeParse.MyDate_.prototype.seconds;

	/**
	 * The date's milliseconds.
	 * 
	 * @type {number?}
	 */
	DateTimeParse.MyDate_.prototype.milliseconds;

	/**
	 * The date's timezone offset.
	 * 
	 * @type {number?}
	 */
	DateTimeParse.MyDate_.prototype.tzOffset;

	/**
	 * The date's day of week. Sunday is 0, Saturday is 6.
	 * 
	 * @type {number?}
	 */
	DateTimeParse.MyDate_.prototype.dayOfWeek;

	/**
	 * 2 digit year special handling. Assuming for example that the
	 * defaultCenturyStart is 6/18/1903. This means that two-digit years will be
	 * forced into the range 6/18/1903 to 6/17/2003. As a result, years 00, 01,
	 * and 02 correspond to 2000, 2001, and 2002. Years 04, 05, etc. correspond
	 * to 1904, 1905, etc. If the year is 03, then it is 2003 if the other
	 * fields specify a date before 6/18, or 1903 if they specify a date
	 * afterwards. As a result, 03 is an ambiguous year. All other two-digit
	 * years are unambiguous.
	 * 
	 * @param {number}
	 *            year 2 digit year value before adjustment.
	 * @return {number} disambiguated year.
	 * @private
	 */
	DateTimeParse.MyDate_.prototype.setTwoDigitYear_ = function(year) {
		var now = new Date();
		var defaultCenturyStartYear = now.getFullYear()
				- DateTimeParse.ambiguousYearCenturyStart;
		var ambiguousTwoDigitYear = defaultCenturyStartYear % 100;
		this.ambiguousYear = (year == ambiguousTwoDigitYear);
		year += Math.floor(defaultCenturyStartYear / 100) * 100
				+ (year < ambiguousTwoDigitYear ? 100 : 0);
		return this.year = year;
	};

	/**
	 * Based on the fields set, fill a Date object. For those fields that not
	 * set, use the passed in date object's value.
	 * 
	 * @param {Date}
	 *            date Date object to be filled.
	 * @param {boolean}
	 *            validation If true, input string will be checked to make sure
	 *            it is valid.
	 * 
	 * @return {boolean} false if fields specify a invalid date.
	 * @private
	 */
	DateTimeParse.MyDate_.prototype.calcDate_ = function(date,
			validation) {
		// year 0 is 1 BC, and so on.
		if (this.era != undefined && this.year != undefined && this.era == 0
				&& this.year > 0) {
			this.year = -(this.year - 1);
		}

		if (this.year != undefined) {
			date.setFullYear(this.year);
		}

		// The setMonth and setDate logic is a little tricky. We need to make
		// sure
		// day of month is smaller enough so that it won't cause a month switch
		// when
		// setting month. For example, if data in date is Nov 30, when month is
		// set
		// to Feb, because there is no Feb 30, JS adjust it to Mar 2. So Feb 12
		// will
		// become Mar 12.
		var orgDate = date.getDate();
		date.setDate(1); // every month has a 1st day, this can actually be
							// anything
		// less than 29.

		if (this.month != undefined) {
			date.setMonth(this.month);
		}

		if (this.day != undefined) {
			date.setDate(this.day);
		} else {
			date.setDate(orgDate);
		}

		if (this.hours == undefined) {
			this.hours = date.getHours();
		}

		// adjust ampm
		if (this.ampm != undefined && this.ampm > 0) {
			if (this.hours < 12) {
				this.hours += 12;
			}
		}
		date.setHours(this.hours);

		if (this.minutes != undefined) {
			date.setMinutes(this.minutes);
		}

		if (this.seconds != undefined) {
			date.setSeconds(this.seconds);
		}

		if (this.milliseconds != undefined) {
			date.setMilliseconds(this.milliseconds);
		}

		// If validation is needed, verify that the uncalculated date fields
		// match the calculated date fields. We do this before we set the
		// timezone offset, which will skew all of the dates.
		//
		// Don't need to check the day of week as it is guaranteed to be
		// correct or return false below.
		if (validation
				&& (this.year != undefined && (this.year != date.getFullYear()
							|| this.year < DateTimeParse.minValidYear 
							|| this.year > DateTimeParse.maxValidYear)
						|| this.month != undefined && this.month != date.getMonth()
						|| this.day != undefined && this.day != date.getDate()
						|| this.hours >= 24 || this.minutes >= 60
						|| this.seconds >= 60 || this.milliseconds >= 1000)) {
			return false;
		}

		// adjust time zone
		if (this.tzOffset != undefined) {
			var offset = date.getTimezoneOffset();
			date.setTime(date.getTime() + (this.tzOffset - offset) * 60 * 1000);
		}

		// resolve ambiguous year if needed
		if (this.ambiguousYear) { // the two-digit year == the default start
									// year
			var defaultCenturyStart = new Date();
			defaultCenturyStart.setFullYear(defaultCenturyStart.getFullYear()
					- DateTimeParse.ambiguousYearCenturyStart);
			if (date.getTime() < defaultCenturyStart.getTime()) {
				date.setFullYear(defaultCenturyStart.getFullYear() + 100);
			}
		}

		// dayOfWeek, validation only
		if (this.dayOfWeek != undefined) {
			if (this.day == undefined) {
				// adjust to the nearest day of the week
				var adjustment = (7 + this.dayOfWeek - date.getDay()) % 7;
				if (adjustment > 3) {
					adjustment -= 7;
				}
				var orgMonth = date.getMonth();
				date.setDate(date.getDate() + adjustment);

				// don't let it switch month
				if (date.getMonth() != orgMonth) {
					date.setDate(date.getDate() + (adjustment > 0 ? -7 : 7));
				}
			} else if (this.dayOfWeek != date.getDay()) {
				return false;
			}
		}
		return true;
	};
	
	jef.date.JavaDateTimeParse = DateTimeParse;

})(jef, jef.i18n);

//
// Date Time Provider
//

(function(jef, Format, Parse){

	if(jef.isDef(jef.date.JavaDateTimeFormatProvider))
		return;
	
	/**
	 * Construct a DateTimeProvider based on current locale.
	 * 
	 * @param {string|number} pattern Pattern specification or pattern type.
	 * @param {object} Options : 
	 *  - timeZone : {jef.i18n.TimeZone} The default time zone for the instance,
	 * 	- locale : {string} locale key, ex: 'en', 'fr_FR', ...  (override default locale)
	 * @constructor
	 */	 
	var DateTimeProvider = function(pattern, options){
		
		var parser = new Parse(pattern, options);
		var formatter = new Format(pattern, options);
		
		/**
		 * Parse the given string and fill info into date object. This version does
		 * not validate the input.
		 * 
		 * @param {string} text The string being parsed.
		 * @param {object} options Options for parsing : 
		 * 		- date {Date} The Date object to hold the parsed date.   
		 * 		- pos {Array.<number>=} Position to pass in and get back. It not given, 
		 * 			process will assume that text end must be reach at the end of parse process
		 * 		- valid {boolean} If process must valid date (ie : date 30/02/2010 is not valid) 
		 * @return {Date} parsed date or null (if error).
		 */
		this.parse = function(text, options){
			return parser.parse(text, options);
		};
		
		this.format = function(date, options){
			return formatter.format(date, options);
		};		
	};
	jef.date.setProvider(DateTimeProvider);	
	
	/**
	* Store provider
	*/
	jef.date.JavaDateTimeFormatProvider = DateTimeProvider;
	
	/**
	 * Static date format
	 */	
	jef.date.addStaticFormatter('EEE MMM dd HH:mm:ss zzz yyyy', { key : 'javaUtilDate', locale : 'en_US' });
	jef.date.addStaticFormatter('yyyy-MM-dd HH:mm:ss.SSS', { key : 'javaSqlTimestamp' });
	jef.date.addStaticFormatter('yyyy-MM-dd', { key : 'javaSqlDate' });
			
})(jef, jef.date.JavaDateTimeFormat, jef.date.JavaDateTimeParse);
